      SUBROUTINE PROPS(R,Z,T,SVF,PCO2,PH2O,PAC,PSC)
C THIS ROUTINE PROVIDES LOCAL (R,Z) CONDITIONS FOR THE MEDIUM
C T     = TEMPERATURE (K)
C SVF   = SOOT VOLUME FRACTION
C PCO2  = CO2 PARTIAL PRESSURE (BAR)
C PH2O  = H2O PARTIAL PRESSURE (BAR)
C PAC   = PARTICLE ABSORPTION COEFFICIENT (1/CM)
C PSC   = PARTICLE SCATTERING COEFFICIENT (1/CM)
C CALCULATES ABSCO (IN 1/CM)
C
      COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN
      SVF=0.
      T=1000.
      PH2O=.1
      PCO2=.1
      PAC=.01
      PSC=0.
      RETURN
      END

      FUNCTION ABSCO(SVF,PCO2,PH2O,PAC,W,T)
C THIS FUNCTION PROVIDES LOCAL, SPECTRAL ABSORPTION COEFFICIENT AS FUNCTION
C OF LOCAL CONDITIONS; USES WIDE-BAND MODEL WITH STRONG OVERLAP (BETA>1)
C SVF   = SOOT VOLUME FRACTION
C PCO2  = CO2 PARTIAL PRESSURE (BAR)
C PH2O  = H2O PARTIAL PRESSURE (BAR)
C PAC   = PARTICLE ABSORPTION COEFFICIENT (1/CM)
C W     = WAVELENGTH (MICRO-M)
C T     = TEMPERATURE (K)
C CALCULATES ABSCO (IN 1/CM)
C
      REAL C1(6),C3(6),CWN(6) 
      COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN
C GRAY PARTICLE BACKGROUND
      ABSCO=PAC
C FOR GRAY MEDIUM (NO GAS OR SOOT) RETURN
      IF(W.LT.0.) RETURN
      WN=1.E4/W
C ****** H20 AND CO2 *******      
      RCO2=1.8616E-6
      RH2O=4.5485E-6
      CWN(1)=3660.
      CWN(2)=2410.
      CWN(3)= 667.
      CWN(4)=3760.
      CWN(5)=1600.
      CWN(6)= 140.
      S=SQRT(T/100.)
      C3(1)=23.5*S
      C3(2)=11.2*S
      C3(3)=12.7*S
      C3(4)=60.0*S
      C3(5)=56.4*S
      C3(6)=69.3*S
      RHO=PCO2/(RCO2*T)
      C1(1)=.04*(1.+.0025*((T/100.)**2-1.))*RHO
      C1(2)=1.1*RHO
      C1(3)=.19*RHO
      RHO=PH2O/(RH2O*T)
      C1(4)=.259*RHO
      C1(5)=.412*RHO
      C1(6)=.0545*EXP(9.*(1.-1./S))*RHO
C
C ****** SOOT PARTICLES ******
      PI36=113.0973
      STNSQ=STN*STN
      STKSQ=STK*STK
      SOOTF=PI36*STN*STK*SVF/((STNSQ-STKSQ+2.)**2+4.*STNSQ*STKSQ)
      ABSCO=ABSCO+SOOTF*WN
C ****** H20 AND CO2 ******
      DO 10 I=1,6
         IF(I.NE.2) THEN
            ABSCO=ABSCO+C1(I)/C3(I)*EXP(-2.*ABS(WN-CWN(I))/C3(I))
         ELSEIF(WN.LT.CWN(I)) THEN
            ABSCO=ABSCO+C1(I)/C3(I)*EXP((WN-CWN(I))/C3(I))
       ENDIF
  10    CONTINUE
      RETURN
      END

      FUNCTION RNUM(L)
      INTEGER X(7),A(7),C(7)
      DATA X/25,1,12,0,0,0,0/
      DATA A/13,12,18,26,15,13,28/
      DATA C/13,26,5,22,12,24,6/
      DO 10 I=1,7
      IR=8-I
      ISUM=C(IR)
      DO 5 J=1,IR
      JR=IR+1-J
   5  ISUM=ISUM+X(JR)*A(J)
  10  X(IR)=ISUM
      DO 20 I=1,6
      IT=X(I)/32
      X(I)=X(I)-32*IT
  20  X(I+1)=X(I+1)+IT
      IT=X(7)/32
      X(7)=X(7)-32*IT
      RNUM=((((X(4)+.5)/32.+X(5))/32.+X(6))/32.+X(7))/32.
      RETURN
      END
