      PROGRAM RNARRAY
C THIS ROUTINE EVALUATES RANDOM-NUMBER RELATIONSHIPS FOR LOCATION (R,Z),
C AND FOR WAVELENGTHS; ALSO EVALUATES TOTAL EMITTED ENERGY (W) 
C AND CENTER-LINE PLANCK-MEAN ABSORPTION COEFFICIENTS.
C RUNNING THIS CODE REQUIRES:
C -- USER-SUPPLIED SUBROUTINE NAMED 'PROPS' AND FUNCTION 'ABSCO'
C
C SUBROUTINE PROPS(R,Z,T,SVF,PCO2,PH2O,PAC,PSC) PROVIDES LOCAL (R,Z) CONDITIONS FOR THE MEDIUM
C  INPUT:
C   R     = RADIAL COORDINATE (CM)
C   Z     = AXIAL COORDINATE (CM)
C  OUTPUT:
C   T     = TEMPERATURE (K)
C   SVF   = SOOT VOLUME FRACTION
C   PCO2  = CO2 PARTIAL PRESSURE (BAR)
C   PH2O  = H2O PARTIAL PRESSURE (BAR)
C   PAC   = PARTICLE ABSORPTION COEFFICIENT (1/CM)
C   PSC   = PARTICLE SCATTERING COEFFICIENT (1/CM)
C  NOTE: THIS ROUTINE REQUIRES THE COMMON STATEMENT 'COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN'
C
C FUNCTION ABSCO(SVF,PCO2,PH2O,PAC,W,T)  CALCULATES THE LOCAL, SPECTRAL ABSORPTION 
C COEFFICIENT AS FUNCTION OF LOCAL CONDITIONS; 
C  INPUT:
C   SVF   = SOOT VOLUME FRACTION
C   PCO2  = CO2 PARTIAL PRESSURE (BAR)
C   PH2O  = H2O PARTIAL PRESSURE (BAR)
C   PAC   = PARTICLE ABSORPTION COEFFICIENT (1/CM)
C   W     = WAVELENGTH (MICRO-M)
C   T     = TEMPERATURE (K)
C  OUTPUT:
C   ABSCO = LOCAL, SPECTRAL ABSORPTION COEFFICIENT (IN 1/CM)
C  NOTES: THIS ROUTINE REQUIRES THE COMMON STATEMENT 'COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN'
C         FOR THE CASE W<0 THE ROUTINE SHOULD RETURN A GRAY ABSORPTION COEFFICIENT
C
C JUNE/JULY 1981; RECODED AUGUST 6,1987; UPDATED APRIL 2003
      PARAMETER (NRP=21,NZP=21,NNP=101,NS=MAX(NRP,NZP))
      REAL R(NZP),Z(NRP),WAVE(NNP),WVE2(NRP,NZP,NNP),B(NS),C(NS),D(NS)
      REAL RRA(NNP),ZRINT(NZP),ZRI(NZP),ZR(NRP,NNP),ENPR(NRP),ENINT(NRP)
      REAL PLMCL(NZP)
      COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN
      OPEN(UNIT=7,FILE='datlam.dat',STATUS='UNKNOWN')
C******************************************************************************
C INPUT:
C NRP   = NO OF RADIAL NODES FOR MEDIUM EMISSION RANDOM NUMBER RELATIONSHIPS
C NZP   = NO OF AXIAL NODES FOR MEDIUM EMISSION RANDOM NUMBER RELATIONSHIPS
C NNP   = NO OF RANDOM NUMBERS FOR MEDIUM EMISSION RELATIONSHIPS
C RL    = RADIUS OF CYLINDER, (CM)
C ZL    = LENGTH OF CYLINDER, (CM)
C AN    = REFRACTIVE INDEX OF MEDIUM (AN=1.0 FOR GASES)
C STN   = REFRACTIVE INDEX OF SOOT
C STK   = ABSORPTIVE INDEX OF SOOT
C IGRAY = GRAY/NONGRAY MEDIUM SWITCH: IGRAY=0 NONGRAY, 
C         IGRAY=1 GRAY (IGNORING CONTRIBUTION FROM GASES; ABS.COEFF=PAC)
C LU    = LOGICAL UNIT NUMBER FOR OUTPUT: LU=6 SENDS OUTPUT TO SCREEN,
C         OTHER (LEGAL) VALUES SEND OUTPUT TO FILE datlam.dat
C******************************************************************************
C OUTPUT:
C FILE 'datlam.dat' CONTAINS RANDOM NUMBER RELATIONSHIPS GENERATED BY 'RNARRAY'
C ETOTAL           = TOTAL ENERGY EMITTED (PER UNIT TIME) BY ENTIRE VOLUME
C PLMCL(I),I=1,NZP = PLANCK-MEAN ABSORPTION COEFFICIENT ALONG CENTERLINE
C RRA(J),J=1,NNP   = EMISSION RADIUS AS f(RANDOM#)
C ZR(K,J),K=1,NRP,J=1,NNP = EMISSION Z-LOCATION AS f(r,RANDOM#)
C WVE2(K,I,J),K=1,NRP,I=1,NZP,J=1,NRNP1 = EMISSION WAVELENGTH AS f(r,z,RANDOM#)
C                                         (IGRAY=0 ONLY)
C******************************************************************************
C RL=radius (cm), ZL=length (cm), AN=refractive index of medium (-), 
C STN= soot refractive index, STK= soot absorptive index, 
C NRR=# radial nodes, NZL=# axial nodes, NRN=# wavelength nodes
C
      LU=7
      IGRAY=0
      RL=10.
      ZL=10.
      AN=1.
      STN=1.89
      STK=0.92
      NRR=NRP
      NZL=NZP
      NRN=NNP-1
      NRNP1=NRN+1
      DR=RL/(NRR-1.)
      DZ=ZL/(NZL-1.)
C
      DO 100 K=1,NRR
         R(K)=(K-1.)*DR
C
         DO 10 I=1,NZL
            Z(I)=(I-1.)*DZ
C fluid properties at this location
            CALL PROPS(R(K),Z(I),TIK,FIK,P1IK,P2IK,AC,SC)
C find relationship for WAVE(random-#)
C NONGRAY
            IF(IGRAY.NE.1) THEN
               CALL RLAM(FIK,AC,P1IK,P2IK,TIK,WAVE,PLMAC)
               DO 20 J=1,NRNP1
C WVE2=wvlength(r,z,R#)
   20          WVE2(K,I,J)=WAVE(J)  
            ELSE
C gray
               PLMAC=AC
            ENDIF
            IF(K.EQ.1) PLMCL(I)=PLMAC
C ZRINT = k_Pl * T^4
   10       ZRINT(I)=PLMAC*TIK**4
C fit spline through z-integrand, then integrate
         CALL SPLINE(NZL,Z,ZRINT,B,C,D)
         NZLM1=NZL-1
         ZRI(1)=0.
         DO 40 I=1,NZLM1
            ZRI(I+1)=ZRI(I)+(((D(I)/4.*DZ+C(I)/3.)*DZ+B(I)/2.)*DZ
     1                     +ZRINT(I))*DZ
            IF(ZRI(I+1).GT.ZRI(I)) GOTO 40
            ZRI(I+1)=ZRI(I)+(ZRINT(I)+ZRINT(I+1))*DZ/2.
   40    CONTINUE
C ENPR = INTEGRAL_0^L (4n^2 sigma k_Pl T^4 dz) 
C ZRI  = R_z from eq(20.62c)
         ENPR(K)=4.*AN*AN*5.670E-12*ZRI(NZL)
         DO 50 I=2,NZLM1
   50    ZRI(I)=ZRI(I)/ZRI(NZL)
         ZRI(NZL)=1.000001
C fit monotonically increasing spline through inverse function
         CALL SPLMI(NZL,ZRI,Z,B,C,D)
C evaluate  z(r,random-#) for equidistant random-#
         ZR(K,1)=0.
         JZ=1
         DO 80 IZ=1,NRN
            RZ=IZ/FLOAT(NRN)
   60       IF(ZRI(JZ).GE.RZ) GOTO 70
            JZ=JZ+1
            GOTO 60
   70       DELZ=RZ-ZRI(JZ-1)
   80    ZR(K,IZ+1)=Z(JZ-1)+((D(JZ-1)*DELZ+C(JZ-1))*DELZ+B(JZ-1))*DELZ
  100 CONTINUE
C
C fit spline through r-integrand, then integrate
      CALL SPLINE(NRR,R,ENPR,B,C,D)
      ENINT(1)=0.
      NRM1=NRR-1
C ENINT = INTEGRAL_0^r [INTEGRAL_0^L (4n^2 sigma k_Pl T^4 dz)] r dr
      DO 110 K=1,NRM1
  110    ENINT(K+1)=ENINT(K)+(((D(K)*(DR/5.+R(K)/4.)*DR
     1                      +C(K)*(DR/4.+R(K)/3.))*DR
     2                      +B(K)*(DR/3.+R(K)/2.))*DR
     3                      +ENPR(K)*(DR/2.+R(K)))*DR
C ETOTAL = INTEGRAL_V (4n^2 sigma k_Pl T^4 dz) dV
      ETOTAL=6.2832*ENINT(NRR)
C ENINT = R_r from eq(20.62b)
      DO 120 K=2,NRM1
  120 ENINT(K)=ENINT(K)/ENINT(NRR)
      ENINT(NRR)=1.000001
C fit monotonically increasing spline through inverse function
      CALL SPLMI(NRR,ENINT,R,B,C,D)
C evaluate r(random-#) for equidistant random-#
      RRA(1)=0.
      JA=1
      DO 150 IA=1,NRN
         RA=IA/FLOAT(NRN)
  130    IF(ENINT(JA).GE.RA) GOTO 140
         JA=JA+1
         GOTO 130
  140    DELR=RA-ENINT(JA-1)
  150 RRA(IA+1)=R(JA-1)+((D(JA-1)*DELR+C(JA-1))*DELR+B(JA-1))*DELR
C
      WRITE(LU,200) ETOTAL
      WRITE(LU,200) (PLMCL(I),I=1,NZL)
      WRITE(LU,200) (RRA(J),J=1,NRNP1)
      DO 170 K=1,NRR
         WRITE(LU,200) (ZR(K,J),J=1,NRNP1)
         IF(IGRAY.EQ.1) GOTO 170
         DO 160 I=1,NZL
            WRITE(LU,200) (WVE2(K,I,J),J=1,NRNP1)
  160    CONTINUE
  170 CONTINUE
  200 FORMAT(1X,5E13.6)
      STOP
      END

      SUBROUTINE RLAM(SVF,PAC,PCO2,PH2O,TAV,WAVE,PLAC)
C MAY 27, 1981; RECODED AUGUST 6,1987; UPDATED APRIL 2003
C THIS ROUTINE CALCULATES WAVELENGTHS AS FUNCTION OF RANDOM NUMBERS,
C STORED IN ARRAY WAVE(NRN RAN.#'S) FOR GIVEN LOCAL CONDITIONS;
C ALSO CALCULATES PLANCK-MEAN ABS.COEFF. (PLAC)
      REAL RLM(501),WAVE(NRN+1)
      COMMON RL,ZL,AN,STN,STK,NRR,NZL,NRN
C
C emissive power function: ALT=LAMBDA*TEMP
      EB(ALT)=(.001*ALT)**(-5)/(EXP(C2/ALT)-1.)
      C2=14388.
C
C calculate integral (=RLM) 0 TO LAMBDA over ABSCO*EM.POWER
C
      RLM(1)=0.
      THIRD=0.
      DO 100 IL=1,500
         FIRST=THIRD
         SLAM=(IL-.5)*.04
         SLT=SLAM*TAV
         IF(SLT.LT.180.) SLT=180.
         SECOND=4.*ABSCO(SVF,PCO2,PH2O,PAC,SLAM,TAV)*EB(SLT)
         TLAM=IL*.04
         TLT=TLAM*TAV
         IF(TLT.LT.180.) TLT=180.
         THIRD=ABSCO(SVF,PCO2,PH2O,PAC,TLAM,TAV)*EB(TLT)
  100 RLM(IL+1)=RLM(IL)+FIRST+SECOND+THIRD
C Planck-mean absorption coefficient
      PLAC=.043536*TAV*RLM(501)
C
C normalize integral to give random number RLM
C
      DO 200 IL=2,501
  200 RLM(IL)=RLM(IL)/RLM(501)
      RLM(501)=1.000001
C
C invert function to LAMBDA as function of random number
C
      WAVE(1)=0.
      JW=1
      DO 300 IW=1,NRN
         RW=IW/FLOAT(NRN)
  250    IF(RLM(JW).GE.RW) GOTO 300
         JW=JW+1
         GOTO 250
  300 WAVE(IW+1)=.04*(JW-1.-(RLM(JW)-RW)/(RLM(JW)-RLM(JW-1)))
      RETURN
      END
C      
